from flask import Blueprint, request, current_app
import paho.mqtt.client as mqtt
from escpos.printer import Network
import click
from time import sleep
from json import loads

from ..models.ticket_model import Ticket

bp = Blueprint("services", __name__)


def handle_mqtt_connect(mqttc, userdata, flags, rc):
    current_app.logger.info("Connected on rc " + str(rc) + ".")
    mqttc.subscribe("/printserver/#", 2)
    pass


def handle_mqtt_message(mosq, userdata, msg):
    current_app.logger.info(msg.topic + " " + str(msg.qos) + " " + str(msg.payload))


def handle_printer_0(mosq, userdata, msg):
    return handle_printer(msg)


def handle_printer_1(mosq, userdata, msg):
    return handle_printer(msg)


def handle_printer_2(mosq, userdata, msg):
    return handle_printer(msg)


def handle_printer_3(mosq, userdata, msg):
    return handle_printer(msg)


def handle_printer_4(mosq, userdata, msg):
    return handle_printer(msg)


def handle_printer_5(mosq, userdata, msg):
    return handle_printer(msg)


def handle_printer_6(mosq, userdata, msg):
    return handle_printer(msg)


def handle_printer_7(mosq, userdata, msg):
    return handle_printer(msg)


def handle_printer_8(mosq, userdata, msg):
    return handle_printer(msg)


def handle_printer_9(mosq, userdata, msg):
    return handle_printer(msg)


def handle_printer(msg):
    content = loads(msg.payload.decode("utf-8"))
    printer_ip = content["imprimanteIp"]

    printer = connect_printer(printer_ip)
    if printer == None:
        default_printer_ip = content["caisseIp"]
        default_printer = connect_printer(default_printer_ip)

        if default_printer == None:
            return False
        else:
            default_printer.set(align="center")
            default_printer.text(
                "*** ERROR ON PRINTER: "
                + str(content["imprimanteId"])
                + "/"
                + str(content["imprimanteIp"])
                + " ***\n\n"
            )
            return send_to_printer(default_printer, content)
    else:
        return send_to_printer(printer, content)


def connect_printer(printer_ip):
    try:
        printer = Network(
            printer_ip,
            timeout=current_app.config["PRINTER_TIMEOUT"],
        )

        return printer

    except Exception as ex:
        current_app.logger.error(ex)
        return None


def send_to_printer(printer, content):
    try:
        ticket = Ticket()
        ticket.from_json(content)
        ticket.to_printer(printer)

        return True
    except Exception as ex:
        current_app.logger.error(ex)
        return False


@bp.cli.command("print")
def print_command():
    """
    Reads from Mqtt topic, processes data and sends it to printer.

    $ flask services print
    """
    mqttc = mqtt.Client("services")

    mqttc.message_callback_add("/printserver/printer0/#", handle_printer_0)
    mqttc.message_callback_add("/printserver/printer1/#", handle_printer_1)
    mqttc.message_callback_add("/printserver/printer2/#", handle_printer_2)
    mqttc.message_callback_add("/printserver/printer3/#", handle_printer_3)
    mqttc.message_callback_add("/printserver/printer4/#", handle_printer_4)
    mqttc.message_callback_add("/printserver/printer5/#", handle_printer_5)
    mqttc.message_callback_add("/printserver/printer6/#", handle_printer_6)
    mqttc.message_callback_add("/printserver/printer7/#", handle_printer_7)
    mqttc.message_callback_add("/printserver/printer8/#", handle_printer_8)
    mqttc.message_callback_add("/printserver/printer9/#", handle_printer_9)

    # Add message callbacks that will only trigger on a specific subscription match.
    mqttc.on_connect = handle_mqtt_connect
    mqttc.on_message = handle_mqtt_message

    current_app.logger.info("Mqtt Client connecting ...")
    try:
        mqttc.connect(
            current_app.config["MQTT_BROKER_URL"],
            current_app.config["MQTT_BROKER_PORT"],
            current_app.config["MQTT_KEEPALIVE"],
        )
        current_app.logger.info("Mqtt Client connected!")
    except Exception as ex:
        current_app.logger.error(ex)

    mqttc.loop_forever()
